import ooo.connector.BootstrapSocketConnector;

import com.sun.star.beans.PropertyValue;
import com.sun.star.beans.XPropertySet;
import com.sun.star.bridge.XUnoUrlResolver;
import com.sun.star.document.XDocumentInsertable;
import com.sun.star.frame.*;
import com.sun.star.lang.XMultiComponentFactory;
import com.sun.star.lang.XComponent;
import com.sun.star.lang.XMultiServiceFactory;
import com.sun.star.style.BreakType;
import com.sun.star.text.*;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;
import com.sun.star.util.XCloseable;
import java.io.*;
import java.net.*;
import java.nio.file.*;


public class DocumentConversionTest{
	
	static String programDirWin = "C:/Program Files/LibreOffice/program";
    static String programDirLinux = "/opt/libreoffice6.1/program";
    static String programDir;
    
    static int oooPortWin =8299;
    static int oooPortLinux=8199;    
	
	static String sourceFile = "/sourceDocument.xhtml";
			
	static String sConnectionString1 = "uno:socket,host=localhost,port=";
	static String sConnectionString2 = ";urp;StarOffice.ServiceManager"; 
	static String sConnectionString ;
	
	public static void main(String args[]){
		int runCount = 1;
		try{
		
		
		// determine our environment
		String osName = System.getProperty("os.name");
		if (osName.contains("Windows")){
			programDir = programDirWin;
			sConnectionString = sConnectionString1 + oooPortWin + sConnectionString2;
		} else {
			programDir = programDirLinux;
			sConnectionString = sConnectionString1 + oooPortLinux + sConnectionString2;
		}

		
		// find current, source directories
		File currentDir = new File(System.getProperty("user.dir")); // current working dir
		File sourceDir = new File(currentDir, "source");
		File sourceFile = new File(sourceDir, "sourceDocument2.xhtml");
		Path sourcePath = sourceFile.toPath();
		
		boolean success = true;
		while (success){
		    // create new directory for this run
		    String dirName = String.format("%05d" , runCount);
		    File resultDir = new File(currentDir, "results/"+dirName);
		    resultDir.mkdirs();
		    
		    // copy source file		    
		    Path sourceFileCopy = Files.copy(sourcePath, resultDir.toPath().resolve(sourcePath.getFileName()));
		    
		    // execute conversion
		    URI sourceFileCopyUri = sourceFileCopy.toUri();
		    File outputFile = new File(resultDir, "resultDocument.pdf");
		    success = performConversion(sourceFileCopyUri.toString(), outputFile.toURI().toString());
		    runCount++;
		    System.out.print('.');
		    
		    if (runCount > 10000){
		    	System.out.println("completed 10000 iterations");
		    	success = false;
		    }
		}
		} catch (Exception e){e.printStackTrace();}
		System.out.println("DocumentConversionTest terminated with "+runCount+" documents converted");
	}
	
	public static boolean performConversion(String inURL, String outURL){
				
		boolean success = false;
		
		try{
		    XComponentContext xContext = BootstrapSocketConnector.bootstrap(programDir);
		    XComponentContext xComponentContext = com.sun.star.comp.helper.Bootstrap.createInitialComponentContext( null );
            
            XMultiComponentFactory xMultiComponentFactory = xComponentContext.getServiceManager();
            
            Object objectUrlResolver = xMultiComponentFactory.createInstanceWithContext("com.sun.star.bridge.UnoUrlResolver", xComponentContext );
            
            XUnoUrlResolver xURLResolver = ( XUnoUrlResolver )
               UnoRuntime.queryInterface( XUnoUrlResolver.class,
               objectUrlResolver );
               
            Object initialContext = xURLResolver.resolve(sConnectionString);
            
            XMultiServiceFactory xSMGR = (com.sun.star.lang.XMultiServiceFactory)UnoRuntime.queryInterface(
                    com.sun.star.lang.XMultiServiceFactory.class,
                    initialContext); 
            
		    com.sun.star.frame.XComponentLoader xcomponentloader = (com.sun.star.frame.XComponentLoader)UnoRuntime.queryInterface(
                com.sun.star.frame.XComponentLoader.class,
                xSMGR.createInstance("com.sun.star.frame.Desktop"));  
            PropertyValue[] loadProps = new PropertyValue[0];    
            
		    XComponent objectDocument = xcomponentloader.loadComponentFromURL(inURL, "_blank", 0, loadProps);
		    
		    // export as pdf
		    XStorable xstorable =( XStorable ) UnoRuntime.queryInterface( XStorable.class,objectDocument );        
          	
          PropertyValue propertyvalue[] = new PropertyValue[ 2 ];
          propertyvalue[ 0 ] = makePropertyValue("Overwrite", new Boolean(true));          
          propertyvalue[ 1 ] = makePropertyValue("FilterName", "writer_pdf_Export"); 
          
          // Storing and converting the document
          xstorable.storeToURL( outURL, propertyvalue );                    
          
          // Getting the method close() for closing the document
          XCloseable xcomponent = ( XCloseable ) UnoRuntime.queryInterface( XCloseable.class, xstorable );
          
          // Closing the converted document          
          xcomponent.close(true);
          success = true;
		} catch (Exception e){
			e.printStackTrace();
		}
		return success;
	}
	
    public static PropertyValue makePropertyValue(String name, Object value){
        if (name == null){
            throw new IllegalArgumentException ("Name argument cannot be null");
        }
        PropertyValue pv = new PropertyValue();
        pv.Name = name;
        pv.Value = value;
        return pv;
    }	
}